<?php
// See global.inc.php-dist for full list of configuration options
$glob['adminFile'] = '[[soft_adminFile]]';
$glob['adminFolder'] = '[[soft_adminFolder]]';
$glob['cache'] = 'file';
$glob['cookie_domain'] = '.[[domhost]]';
$glob['dbdatabase'] = '[[softdb]]';
$glob['dbhost'] = '[[softdbhost]]';
$glob['dbpassword'] = '[[softdbpass]]';
$glob['dbport'] = '';
$glob['dbprefix'] = '[[dbprefix]]';
$glob['dbsocket'] = '';
$glob['dbusername'] = '[[softdbuser]]';
$glob['installed'] = '1';
$glob['standard_url'] = '[[softurl]]';
$glob['install_source'] = 'softaculous';
